const fs = require('fs');
const { resolve } = require('path');

const ensureSlash = (path, needsSlash) => {
  const hasSlash = path.endsWith('/');
  if (hasSlash && !needsSlash) {
    return path.substr(path, path.length - 1);
  } else if (!hasSlash && needsSlash) {
    return `${path}/`;
  }
  return path;
};

const resolveSDK = relativePath => {
  return resolve(__dirname, relativePath);
};

const moduleFileExtensions = [
  'web.mjs',
  'mjs',
  'web.js',
  'js',
  'web.ts',
  'ts',
  'web.tsx',
  'tsx',
  'json',
  'web.jsx',
  'jsx'
];

const appDirectory = fs.realpathSync(process.cwd());

const resolveApp = relativePath => {
  return resolve(appDirectory, relativePath);
};

const resolveModule = (filePath) => {
  const extension = moduleFileExtensions.find(ext =>
    fs.existsSync(resolveApp(`${filePath}.${ext}`))
  );
  if (extension) {
    return resolveApp(`${filePath}.${extension}`);
  }
  return resolveApp(`${filePath}.js`);
};

module.exports = {
  appPublic: resolveApp('public'),
  appFavicon: resolveApp('public/favicon.png'),
  appFaviconIco: resolveApp('public/favicon.ico'),
  appPackageJson: resolveApp('package.json'),
  appAbcJson: resolveApp('abc.json'),
  appSrc: resolveApp('src'),
  appIndexJs: resolveModule('src/index'),
  defaultAppHtml: resolveApp('public/index.html'),
  defaultBuildPath: resolveApp('build'),
  defaultPublicPath: ensureSlash('/', true),
  appNodeModules: resolveApp('node_modules'),
  sdkNodeModules: resolveSDK('../../node_modules'),
  resolveApp,
  appDirectory,
};
